unit MainUnit;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, LResources, Forms, Controls, Graphics, Dialogs, ExtCtrls,
  ActnList, Menus, ComCtrls;

type

  { TfrmMain }

  TfrmMain = class(TForm)
    actExit: TAction;
    actNew: TAction;
    actOpen: TAction;
    actSave: TAction;
    lstImages: TImageList;
    lstActions: TActionList;
    imgField: TImage;
    itmNew: TMenuItem;
    itmOpen: TMenuItem;
    itmSave: TMenuItem;
    itmSeparator1: TMenuItem;
    itmExit: TMenuItem;
    mnuEditor: TMenuItem;
    mnuMain: TMainMenu;
    dlgOpen: TOpenDialog;
    dlgSave: TSaveDialog;
    sbarMain: TStatusBar;
    tbarMain: TToolBar;
    btnNew: TToolButton;
    btnOpen: TToolButton;
    btnSave: TToolButton;
    btnSeparator1: TToolButton;
    btnExit: TToolButton;
    procedure actExitExecute(Sender: TObject);
    procedure actNewExecute(Sender: TObject);
    procedure actOpenExecute(Sender: TObject);
    procedure actSaveExecute(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormResize(Sender: TObject);
    procedure imgFieldMouseDown(Sender: TOBject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure imgFieldMouseMove(Sender: TObject; Shift: TShiftState; X,
      Y: Integer);
    procedure imgFieldMouseUp(Sender: TOBject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
  private
    { private declarations }
  public
    { public declarations }
    procedure DrawField;
    function GetCell(AX, AY: Integer): TPoint;
    CellWidth, CellHeight: Single;
    CellValue: Integer;
  end; 

var
  frmMain: TfrmMain;

implementation

uses
  NewUnit, GameUnit;
  
{ TfrmMain }

procedure TfrmMain.FormResize(Sender: TObject);
begin
  imgField.Picture.Bitmap.Width := imgField.Width;
  imgField.Picture.Bitmap.Height := imgField.Height;
  CellWidth := imgField.Width / FieldWidth;
  CellHeight := imgField.Height / FieldHeight;
  DrawField;
end;

procedure TfrmMain.imgFieldMouseDown(Sender: TOBject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
  if Button = mbLeft then
    CellValue := C_Wall
  else
    CellValue := C_Empty;

  imgFieldMouseMove(Sender, Shift, X, Y);
end;

procedure TfrmMain.imgFieldMouseMove(Sender: TObject; Shift: TShiftState; X,
  Y: Integer);
begin
  if CellValue <> -1 then
  begin
    Field[GetCell(X, Y).X, GetCell(X, Y).Y] := CellValue;
    DrawField;
  end;
end;

procedure TfrmMain.imgFieldMouseUp(Sender: TOBject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
  CellValue := -1;
end;

procedure TfrmMain.FormCreate(Sender: TObject);
begin
  DoubleBuffered := True;
  CellValue := -1;
end;

procedure TfrmMain.actNewExecute(Sender: TObject);
begin
  if frmNew.Execute then
  begin
    CreateField(frmNew.FieldWidth, frmNew.FieldHeight);
    FormResize(Self);
  end;
end;

procedure TfrmMain.actExitExecute(Sender: TObject);
begin
  Close;
end;

procedure TfrmMain.actOpenExecute(Sender: TObject);
begin
  if (dlgOpen.Execute) and (LoadField(dlgOpen.FileName)) then
  begin
    FormResize(Self);
  end;
end;

procedure TfrmMain.actSaveExecute(Sender: TObject);
begin
  if dlgSave.Execute then
  begin
    SaveField(dlgSave.FileName);
  end;
end;

procedure TfrmMain.DrawField;
var
  X, Y: Integer;
  
begin
  for Y := 0 to FieldHeight - 1 do
    for X := 0 to FieldWidth - 1 do
    begin
      if Field[X, Y] = C_Empty then
        imgField.Canvas.Brush.Color := clWhite
      else
        imgField.Canvas.Brush.Color := clGray;
      imgField.Canvas.Rectangle(Trunc(X * CellWidth), Trunc(Y * CellHeight),
        Trunc((X + 1) * CellWidth), Trunc((Y + 1) * CellHeight));
    end;
end;

function TfrmMain.GetCell(AX, AY: Integer): TPoint;
begin
  Result.X := Trunc(AX / CellWidth);
  Result.Y := Trunc(AY / CellHeight);
end;

initialization
  {$I mainunit.lrs}

end.

